@@ -74,7 +74,7 @@ struct NautilusCanvasItemDetails {
 	/* The image, text, font. */
 	double x, y;
 	GdkPixbuf *pixbuf;
-	GdkPixbuf *rendered_pixbuf;
+	cairo_surface_t *rendered_surface;
 	char *editable_text;		/* Text that can be modified by a renaming function */
 	char *additional_text;		/* Text that cannot be modifed, such as file size, etc. */
 	GdkPoint *attach_points;
@@ -222,9 +222,9 @@ nautilus_canvas_item_finalize (GObject *object)
 	g_free (details->editable_text);
 	g_free (details->additional_text);
 	g_free (details->attach_points);
-	
-	if (details->rendered_pixbuf != NULL) {
-		g_object_unref (details->rendered_pixbuf);
+
+	if (details->rendered_surface != NULL) {
+		cairo_surface_destroy (details->rendered_surface);
 	}
 
 	if (details->editable_text_layout != NULL) {
@@ -428,17 +428,40 @@ nautilus_canvas_item_get_property (GObject        *object,
 	}
 }
 
+static void
+get_scaled_icon_size (NautilusCanvasItem *item,
+		      gint *width,
+		      gint *height)
+{
+	EelCanvas *canvas;
+	GdkPixbuf *pixbuf = NULL;
+	gint scale;
+
+	if (item != NULL) {
+		canvas = EEL_CANVAS_ITEM (item)->canvas;
+		scale = gtk_widget_get_scale_factor (GTK_WIDGET (canvas));
+		pixbuf = item->details->pixbuf;
+	}
+
+	if (width)
+		*width = (pixbuf == NULL) ? 0 : (gdk_pixbuf_get_width (pixbuf) / scale);
+	if (height)
+		*height = (pixbuf == NULL) ? 0 : (gdk_pixbuf_get_height (pixbuf) / scale);
+}
+
 cairo_surface_t *
 nautilus_canvas_item_get_drag_surface (NautilusCanvasItem *item)
 {
 	cairo_surface_t *surface;
 	EelCanvas *canvas;
 	int width, height;
+	int pix_width, pix_height;
 	int item_offset_x, item_offset_y;
 	EelIRect icon_rect;
 	double item_x, item_y;
 	cairo_t *cr;
 	GtkStyleContext *context;
+	cairo_surface_t *drag_surface;
 	
 	g_return_val_if_fail (NAUTILUS_IS_CANVAS_ITEM (item), NULL);
 
@@ -463,18 +486,24 @@ nautilus_canvas_item_get_drag_surface (NautilusCanvasItem *item)
 	width = EEL_CANVAS_ITEM (item)->x2 - EEL_CANVAS_ITEM (item)->x1;
 	height = EEL_CANVAS_ITEM (item)->y2 - EEL_CANVAS_ITEM (item)->y1;
 
-        surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
-					      width, height);
-
+        surface = gdk_window_create_similar_surface (gtk_widget_get_window (GTK_WIDGET (canvas)),
+						     CAIRO_CONTENT_COLOR_ALPHA,
+						     width, height);
 	cr = cairo_create (surface);
 
-	gtk_render_icon (context, cr, item->details->pixbuf,
-			   item_offset_x, item_offset_y);
+	drag_surface = gdk_cairo_surface_create_from_pixbuf (item->details->pixbuf,
+							     gtk_widget_get_scale_factor (GTK_WIDGET (canvas)),
+							     gtk_widget_get_window (GTK_WIDGET (canvas)));
+	gtk_render_icon_surface (context, cr, drag_surface,
+				 item_offset_x, item_offset_y);
+	cairo_surface_destroy (drag_surface);
+
+	get_scaled_icon_size (item, &pix_width, &pix_height);
 
 	icon_rect.x0 = item_offset_x;
 	icon_rect.y0 = item_offset_y;
-	icon_rect.x1 = item_offset_x + gdk_pixbuf_get_width (item->details->pixbuf);
-	icon_rect.y1 = item_offset_y + gdk_pixbuf_get_height (item->details->pixbuf);
+	icon_rect.x1 = item_offset_x + pix_width;
+	icon_rect.y1 = item_offset_y + pix_height;
 
 	draw_embedded_text (item, cr,
 			    item_offset_x, item_offset_y);
@@ -506,9 +535,9 @@ nautilus_canvas_item_set_image (NautilusCanvasItem *item,
 	if (details->pixbuf != NULL) {
 		g_object_unref (details->pixbuf);
 	}
-	if (details->rendered_pixbuf != NULL) {
-		g_object_unref (details->rendered_pixbuf);
-		details->rendered_pixbuf = NULL;
+	if (details->rendered_surface != NULL) {
+		cairo_surface_destroy (details->rendered_surface);
+		details->rendered_surface = NULL;
 	}
 
 	details->pixbuf = image;
@@ -1236,13 +1265,14 @@ draw_pixbuf (GdkPixbuf *pixbuf,
 }
 
 /* shared code to highlight or dim the passed-in pixbuf */
-static GdkPixbuf *
-real_map_pixbuf (NautilusCanvasItem *canvas_item)
+static cairo_surface_t *
+real_map_surface (NautilusCanvasItem *canvas_item)
 {
 	EelCanvas *canvas;
 	GdkPixbuf *temp_pixbuf, *old_pixbuf;
 	GtkStyleContext *style;
 	GdkRGBA color;
+	cairo_surface_t *surface;
 	
 	temp_pixbuf = canvas_item->details->pixbuf;
 	canvas = EEL_CANVAS_ITEM(canvas_item)->canvas;
@@ -1272,23 +1302,28 @@ real_map_pixbuf (NautilusCanvasItem *canvas_item)
 
 		g_object_unref (old_pixbuf);
 	}
-	
-	return temp_pixbuf;
+
+	surface = gdk_cairo_surface_create_from_pixbuf (temp_pixbuf,
+							gtk_widget_get_scale_factor (GTK_WIDGET (canvas)),
+							gtk_widget_get_window (GTK_WIDGET (canvas)));
+	g_object_unref (temp_pixbuf);
+
+	return surface;
 }
 
-static GdkPixbuf *
-map_pixbuf (NautilusCanvasItem *canvas_item)
+static cairo_surface_t *
+map_surface (NautilusCanvasItem *canvas_item)
 {
-	if (!(canvas_item->details->rendered_pixbuf != NULL
+	if (!(canvas_item->details->rendered_surface != NULL
 	      && canvas_item->details->rendered_is_prelit == canvas_item->details->is_prelit
 	      && canvas_item->details->rendered_is_highlighted_for_selection == canvas_item->details->is_highlighted_for_selection
 	      && canvas_item->details->rendered_is_highlighted_for_drop == canvas_item->details->is_highlighted_for_drop
 	      && canvas_item->details->rendered_is_highlighted_for_clipboard == canvas_item->details->is_highlighted_for_clipboard
 	      && (canvas_item->details->is_highlighted_for_selection && canvas_item->details->rendered_is_focused == gtk_widget_has_focus (GTK_WIDGET (EEL_CANVAS_ITEM (canvas_item)->canvas))))) {
-		if (canvas_item->details->rendered_pixbuf != NULL) {
-			g_object_unref (canvas_item->details->rendered_pixbuf);
+		if (canvas_item->details->rendered_surface != NULL) {
+			cairo_surface_destroy (canvas_item->details->rendered_surface);
 		}
-		canvas_item->details->rendered_pixbuf = real_map_pixbuf (canvas_item);
+		canvas_item->details->rendered_surface = real_map_surface (canvas_item);
 		canvas_item->details->rendered_is_prelit = canvas_item->details->is_prelit;
 		canvas_item->details->rendered_is_highlighted_for_selection = canvas_item->details->is_highlighted_for_selection;
 		canvas_item->details->rendered_is_highlighted_for_drop = canvas_item->details->is_highlighted_for_drop;
@@ -1296,9 +1331,9 @@ map_pixbuf (NautilusCanvasItem *canvas_item)
 		canvas_item->details->rendered_is_focused = gtk_widget_has_focus (GTK_WIDGET (EEL_CANVAS_ITEM (canvas_item)->canvas));
 	}
 
-	g_object_ref (canvas_item->details->rendered_pixbuf);
+	cairo_surface_reference (canvas_item->details->rendered_surface);
 
-	return canvas_item->details->rendered_pixbuf;
+	return canvas_item->details->rendered_surface;
 }
 
 static void
@@ -1368,7 +1403,7 @@ nautilus_canvas_item_draw (EelCanvasItem *item,
 	NautilusCanvasItem *canvas_item;
 	NautilusCanvasItemDetails *details;
 	EelIRect icon_rect;
-	GdkPixbuf *temp_pixbuf;
+	cairo_surface_t *temp_surface;
 	GtkStyleContext *context;
 
 	container = NAUTILUS_CANVAS_CONTAINER (item->canvas);
@@ -1385,12 +1420,12 @@ nautilus_canvas_item_draw (EelCanvasItem *item,
 	gtk_style_context_add_class (context, "nautilus-canvas-item");
 
 	icon_rect = canvas_item->details->icon_rect;
-	temp_pixbuf = map_pixbuf (canvas_item);
+	temp_surface = map_surface (canvas_item);
 
-	gtk_render_icon (context, cr,
-			   temp_pixbuf,
-			   icon_rect.x0, icon_rect.y0);
-	g_object_unref (temp_pixbuf);
+	gtk_render_icon_surface (context, cr,
+				 temp_surface,
+				 icon_rect.x0, icon_rect.y0);
+	cairo_surface_destroy (temp_surface);
 
 	draw_embedded_text (canvas_item, cr, icon_rect.x0, icon_rect.y0);
 	
@@ -1710,6 +1745,7 @@ nautilus_canvas_item_ensure_bounds_up_to_date (NautilusCanvasItem *canvas_item)
 	EelIRect total_rect, total_rect_for_layout, total_rect_for_entire_text;
 	EelCanvasItem *item;
 	double pixels_per_unit;
+	gint width, height;
 	
 	details = canvas_item->details;
 	item = EEL_CANVAS_ITEM (canvas_item);
@@ -1724,17 +1760,13 @@ nautilus_canvas_item_ensure_bounds_up_to_date (NautilusCanvasItem *canvas_item)
 		icon_rect.y0 = 0;
 		icon_rect_raw.x0 = 0;
 		icon_rect_raw.y0 = 0;
-		if (details->pixbuf == NULL) {
-			icon_rect.x1 = icon_rect.x0;
-			icon_rect.y1 = icon_rect.y0;
-			icon_rect_raw.x1 = icon_rect_raw.x0;
-			icon_rect_raw.y1 = icon_rect_raw.y0;
-		} else {
-			icon_rect_raw.x1 = icon_rect_raw.x0 + gdk_pixbuf_get_width (details->pixbuf);
-			icon_rect_raw.y1 = icon_rect_raw.y0 + gdk_pixbuf_get_height (details->pixbuf);
-			icon_rect.x1 = icon_rect_raw.x1 / pixels_per_unit;
-			icon_rect.y1 = icon_rect_raw.y1 / pixels_per_unit;
-		}
+
+		get_scaled_icon_size (canvas_item, &width, &height);
+
+		icon_rect_raw.x1 = icon_rect_raw.x0 + width;
+		icon_rect_raw.y1 = icon_rect_raw.y0 + height;
+		icon_rect.x1 = icon_rect_raw.x1 / pixels_per_unit;
+		icon_rect.y1 = icon_rect_raw.y1 / pixels_per_unit;
 		
 		/* Compute text rectangle. */
 		text_rect = compute_text_rectangle (canvas_item, icon_rect, FALSE, BOUNDS_USAGE_FOR_DISPLAY);
@@ -1759,18 +1791,17 @@ nautilus_canvas_item_get_icon_rectangle (const NautilusCanvasItem *item)
 {
 	EelDRect rectangle;
 	double pixels_per_unit;
-	GdkPixbuf *pixbuf;
+	gint width, height;
 	
 	g_return_val_if_fail (NAUTILUS_IS_CANVAS_ITEM (item), eel_drect_empty);
 
 	rectangle.x0 = item->details->x;
 	rectangle.y0 = item->details->y;
 	
-	pixbuf = item->details->pixbuf;
-	
 	pixels_per_unit = EEL_CANVAS_ITEM (item)->canvas->pixels_per_unit;
-	rectangle.x1 = rectangle.x0 + (pixbuf == NULL ? 0 : gdk_pixbuf_get_width (pixbuf)) / pixels_per_unit;
-	rectangle.y1 = rectangle.y0 + (pixbuf == NULL ? 0 : gdk_pixbuf_get_height (pixbuf)) / pixels_per_unit;
+	get_scaled_icon_size (NAUTILUS_CANVAS_ITEM (item), &width, &height);
+	rectangle.x1 = rectangle.x0 + width / pixels_per_unit;
+	rectangle.y1 = rectangle.y0 + height / pixels_per_unit;
 
 	eel_canvas_item_i2w (EEL_CANVAS_ITEM (item),
 			     &rectangle.x0,
@@ -1791,18 +1822,17 @@ nautilus_canvas_item_get_text_rectangle (NautilusCanvasItem *item,
 	EelIRect text_rectangle;
 	EelDRect ret;
 	double pixels_per_unit;
-	GdkPixbuf *pixbuf;
+	gint width, height;
 	
 	g_return_val_if_fail (NAUTILUS_IS_CANVAS_ITEM (item), eel_drect_empty);
 
 	icon_rectangle.x0 = item->details->x;
 	icon_rectangle.y0 = item->details->y;
 	
-	pixbuf = item->details->pixbuf;
-	
 	pixels_per_unit = EEL_CANVAS_ITEM (item)->canvas->pixels_per_unit;
-	icon_rectangle.x1 = icon_rectangle.x0 + (pixbuf == NULL ? 0 : gdk_pixbuf_get_width (pixbuf)) / pixels_per_unit;
-	icon_rectangle.y1 = icon_rectangle.y0 + (pixbuf == NULL ? 0 : gdk_pixbuf_get_height (pixbuf)) / pixels_per_unit;
+	get_scaled_icon_size (item, &width, &height);
+	icon_rectangle.x1 = icon_rectangle.x0 + width / pixels_per_unit;
+	icon_rectangle.y1 = icon_rectangle.y0 + height / pixels_per_unit;
 
 	measure_label_text (item);
 
@@ -1824,27 +1854,27 @@ nautilus_canvas_item_get_text_rectangle (NautilusCanvasItem *item,
         return ret;
 }
 
-
 /* Get the rectangle of the icon only, in canvas coordinates. */
 static void
 get_icon_rectangle (NautilusCanvasItem *item,
-		      EelIRect *rect)
+		    EelIRect *rect)
 {
-	GdkPixbuf *pixbuf;
+	gint width, height;
 
 	g_assert (NAUTILUS_IS_CANVAS_ITEM (item));
 	g_assert (rect != NULL);
 
+
 	eel_canvas_w2c (EEL_CANVAS_ITEM (item)->canvas,
 			item->details->x,
 			item->details->y,
 			&rect->x0,
 			&rect->y0);
 	
-	pixbuf = item->details->pixbuf;
+	get_scaled_icon_size (item, &width, &height);
 	
-	rect->x1 = rect->x0 + (pixbuf == NULL ? 0 : gdk_pixbuf_get_width (pixbuf));
-	rect->y1 = rect->y0 + (pixbuf == NULL ? 0 : gdk_pixbuf_get_height (pixbuf));
+	rect->x1 = rect->x0 + width;
+	rect->y1 = rect->y0 + height;
 }
 
 void
@@ -2385,13 +2415,7 @@ nautilus_canvas_item_accessible_get_image_size
 	NautilusCanvasItem *item;
 
 	item = NAUTILUS_CANVAS_ITEM (atk_gobject_accessible_get_object (ATK_GOBJECT_ACCESSIBLE (image)));
-
-	if (!item || !item->details->pixbuf) {
-		*width = *height = 0;
-	} else {
-		*width = gdk_pixbuf_get_width (item->details->pixbuf);
-		*height = gdk_pixbuf_get_height (item->details->pixbuf);
-	}
+	get_scaled_icon_size (item, width, height);
 }
 
 static void
@@ -2471,7 +2495,7 @@ nautilus_canvas_item_accessible_get_offset_at_point (AtkText	 *text,
 	char *canvas_text;
 	gboolean have_editable;
 	gboolean have_additional;
-	gint text_offset;
+	gint text_offset, height;
 
 	atk_component_get_extents (ATK_COMPONENT (text), &real_x, &real_y,
                                    &real_width, &real_height, coords);
@@ -2482,7 +2506,8 @@ nautilus_canvas_item_accessible_get_offset_at_point (AtkText	 *text,
 	item = NAUTILUS_CANVAS_ITEM (atk_gobject_accessible_get_object (ATK_GOBJECT_ACCESSIBLE (text)));
 
 	if (item->details->pixbuf) {
-		y -= gdk_pixbuf_get_height (item->details->pixbuf);
+		get_scaled_icon_size (item, NULL, &height);
+		y -= height;
 	}
 	have_editable = item->details->editable_text != NULL &&
 		item->details->editable_text[0] != '\0';
@@ -2578,13 +2603,14 @@ nautilus_canvas_item_accessible_get_character_extents (AtkText	   *text,
 	PangoRectangle rect;
 	PangoRectangle rect0;
 	gboolean have_editable;
-	gint text_offset;
+	gint text_offset, pix_height;
 
 	atk_component_get_position (ATK_COMPONENT (text), &pos_x, &pos_y, coords);
 	item = NAUTILUS_CANVAS_ITEM (atk_gobject_accessible_get_object (ATK_GOBJECT_ACCESSIBLE (text)));
 
 	if (item->details->pixbuf) {
-		pos_y += gdk_pixbuf_get_height (item->details->pixbuf);
+		get_scaled_icon_size (item, NULL, &pix_height);
+		pos_y += pix_height;
 	}
 
 	have_editable = item->details->editable_text != NULL &&
