@@ -2248,7 +2248,7 @@ real_setup_loading_floating_bar (NautilusWindowSlot *slot)
 	nautilus_floating_bar_set_show_spinner (NAUTILUS_FLOATING_BAR (slot->details->floating_bar),
 						TRUE);
 	nautilus_floating_bar_add_action (NAUTILUS_FLOATING_BAR (slot->details->floating_bar),
-					  GTK_STOCK_STOP,
+					  "gtk-stop",
 					  NAUTILUS_FLOATING_BAR_ACTION_ID_STOP);
 
 	gtk_widget_set_halign (slot->details->floating_bar, GTK_ALIGN_END);
@@ -2406,6 +2406,8 @@ nautilus_window_slot_dispose (GObject *object)
 	nautilus_window_slot_clear_forward_list (slot);
 	nautilus_window_slot_clear_back_list (slot);
 
+	nautilus_window_slot_remove_extra_location_widgets (slot);
+
 	if (slot->details->content_view) {
 		nautilus_window_disconnect_content_view (nautilus_window_slot_get_window (slot),
 							 slot->details->content_view);
